﻿B4J=true
Group=Default Group
ModulesStructureVersion=1
Type=Class
Version=7
@EndOfDesignText@
Sub Class_Globals
	Type B4XDateFormatData (FromDate As Long, ThroughDate As Long, IncludeFromThru As Boolean, UseToday As Boolean, DaysFromToday As Int, _
		TextColor As Int, BackColor As Int, FormatFont As B4XFont)
	Public FormatFieldName As String  'If we want to format a field OTHER than the one we are testing then set this - otherwise leave blank
	Private formats As List
	Private xui As XUI
End Sub

Public Sub Initialize
	formats.Initialize
	Dim d As B4XDateFormatData = CreateDefaultFormat
	AddFormatData(d, True, 0)
End Sub

Private Sub CreateDefaultFormat As B4XDateFormatData
	Dim d As B4XDateFormatData
	d.Initialize
	d.UseToday = True
	Return d
End Sub

'Returns a copy of the default format
Public Sub NewFormatData As B4XDateFormatData
	Return CopyFormatData(GetDefaultFormat)
End Sub

'Creates a copy of the given format data.
'This is useful if you want to create a similar format for a different range.
Public Sub CopyFormatData (Data As B4XDateFormatData) As B4XDateFormatData
	Dim d As B4XDateFormatData
	d.Initialize
	d.FromDate = Data.FromDate
	d.ThroughDate = Data.ThroughDate
	d.IncludeFromThru = Data.IncludeFromThru
	d.UseToday = Data.UseToday
	d.DaysFromToday = Data.DaysFromToday
	d.TextColor = Data.TextColor
	d.BackColor = Data.BackColor
	If Data.FormatFont.IsInitialized Then
		#if Not(B4J) OR UI
		d.FormatFont = xui.CreateFont(Data.FormatFont.ToNativeFont, Data.FormatFont.Size)
		#end if
	End If
	Return d
End Sub

'Adds a format data to the formatter. Note that each data object should be used for a single range. Use CopyFormatData to create copies.
'The format datas are stored in a list. When a number is formatted it looks for a data with a matching range. Datas added last will be checked first.
'IncludeEdges - Whether the range is inclusive or not.
Public Sub AddFormatData (Data As B4XDateFormatData, UseToday As Boolean, DaysFromToday As Long)
	Data.UseToday = UseToday
	Data.DaysFromToday = DaysFromToday
	formats.Add(Data)
End Sub
'Returns the default format.
Public Sub GetDefaultFormat As B4XDateFormatData
	Return formats.Get(0)
End Sub

'Gets the format data that matches the passed number.
Public Sub GetFormatData (MDate As Long) As B4XDateFormatData

	For i = formats.Size - 1 To 1 Step - 1
		Dim d As B4XDateFormatData = formats.Get(i)
		If d.UseToday Then 'Test if DAte is ?/>= to todays date +- Days
			Dim DT As Long = DateTime.DateParse(DateTime.Date(DateTime.Now)) + (d.DaysFromToday * DateTime.TicksPerDay)
			If d.IncludeFromThru Then 
				If (MDate >= DT) Then Return d
			Else 
				If (MDate < DT) Then Return d
			End If
		else If (d.FromDate <> 0) And (d.ThroughDate <> 0) Then
			If d.IncludeFromThru Then
				If (MDate >= d.FromDate) And (MDate <= d.ThroughDate) Then Return d
			Else
				If (MDate > d.FromDate) And (MDate < d.ThroughDate) Then Return d
			End If
		Else If (d.FromDate <> 0) Then
			If d.IncludeFromThru Then
				If (MDate >= d.FromDate) Then Return d
			Else
				If (MDate > d.FromDate) Then Return d
			End If
		Else If (d.ThroughDate <> 0) Then
			If d.IncludeFromThru Then
				If (MDate <= d.ThroughDate) Then Return d
			Else
				If (MDate < d.ThroughDate) Then Return d
			End If
		End If

	Next
	Return formats.Get(0)
End Sub


#if Not(B4J) OR UI
'Formats the number and sets it to the text field. Unlike Format, FormatLabel respects the format data font and text size properties.
Public Sub FormatLabel (Number As Long, SrcLabel As B4XView, DestLabel As B4XView)
	If formats.Size = 1 Then
		SrcLabel.Text = Number
		Return   'we only have the default so do nothing
	End If
	Dim data As B4XDateFormatData = GetFormatData(Number)
	If DestLabel.IsInitialized = False Then DestLabel = SrcLabel
	If data.TextColor <> 0 Then DestLabel.TextColor = data.TextColor
	Dim pnl As B4XView = DestLabel.Parent
	If data.BackColor <> 0 Then pnl.Color = data.BackColor
	If data.FormatFont.IsInitialized Then DestLabel.Font = data.FormatFont
End Sub
#end if
